/*
 * Copyright 2011, 2012 Eric Chastain. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ERIC CHASTAIN ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ERIC CHASTAIN OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Eric Chastain.
*/

package me.chastain.chalkup;

import java.util.ArrayList;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class DB 
{ 
    private static final String TAG = "DB";

    private static final String DATABASE_NAME = "chalkup.db";
    private static final int DATABASE_VERSION = 1;

    public final String KEY_TIMESTAMP = "timeStamp";
    
    private static final String PLAYERS_CREATE = "create table players (_id integer primary key autoincrement, dt TIMESTAMP NOT NULL DEFAULT current_timestamp, name text, description text);";
    private static final String GAMES_CREATE = "create table games (_id integer primary key autoincrement, dt TIMESTAMP NOT NULL DEFAULT current_timestamp, name text, description text);";
    private static final String GAME_PLAYERS_CREATE = "create table game_players (_id integer primary key autoincrement, dt TIMESTAMP NOT NULL DEFAULT current_timestamp, game_id integer, player_id integer, player_name string);";
    private static final String TURNS_CREATE = "create table turns (_id integer primary key autoincrement, dt TIMESTAMP NOT NULL DEFAULT current_timestamp, game_players_id integer, change integer);";
    
    private static final boolean logging_enabled = false;

    private final Context context; 
    
    private DatabaseHelper DBHelper;
    private SQLiteDatabase db;

    /**
     * Set up the context of a DB connection.
     */
    public DB(Context ctx) 
    {
        this.context = ctx;
        DBHelper = new DatabaseHelper(context);
    }

    /**
     * The DatabaseHelper class gives us support 
     * for creating and upgrading our SQLite database.
     */
    private static class DatabaseHelper extends SQLiteOpenHelper 
    {
        DatabaseHelper(Context context) { super( context, DATABASE_NAME, null, DATABASE_VERSION ); }

        @Override
        public void onCreate(SQLiteDatabase db) {         	
        	if ( logging_enabled ) {
        		Log.d(TAG, "PLAYERS_CREATE");	
        		Log.d(TAG, "GAMES_CREATE");
        		Log.d(TAG, "GAME_PLAYERS_CREATE");
        		Log.d(TAG, "TURNS_CREATE");
        	}

        	db.execSQL(PLAYERS_CREATE);
        	db.execSQL(GAMES_CREATE);
        	db.execSQL(GAME_PLAYERS_CREATE);
        	db.execSQL(TURNS_CREATE);

        	if (logging_enabled) {
        		Log.d(TAG, "Populating the database with some sample data...");
        	}
        	// some quick test data to avoid having to enter this shit every time we do a deep change...

        	// Sample Games
    		db.execSQL("insert into games(name, description) values ('Scrabble', '');");
    		db.execSQL("insert into games(name, description) values ('Checkers', '');");

        	// Sample Players
    		db.execSQL("insert into players(name, description) values ('Eric', '');");
    		db.execSQL("insert into players(name, description) values ('Hollie', '');");
    		db.execSQL("insert into players(name, description) values ('Jack', '');");
    		db.execSQL("insert into players(name, description) values ('Lilli', '');");

        	// Link players to games
    		db.execSQL("insert into game_players(game_id, player_id, player_name) values(1, 1, '');");
    		db.execSQL("insert into game_players(game_id, player_id, player_name) values(1, 2, '');");
    		db.execSQL("insert into game_players(game_id, player_id, player_name) values(2, 3, '');");
    		db.execSQL("insert into game_players(game_id, player_id, player_name) values(2, 4, '');");

        	// Automatically log some turns
    		db.execSQL("insert into turns(game_players_id, change) values(1, 3);");
    		db.execSQL("insert into turns(game_players_id, change) values(2, 4);");
    		db.execSQL("insert into turns(game_players_id, change) values(1, 1);");
    		db.execSQL("insert into turns(game_players_id, change) values(2, 1);");
    		db.execSQL("insert into turns(game_players_id, change) values(2, -5);");
    		db.execSQL("insert into turns(game_players_id, change) values(3, 100);");
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) 
        {
        	if (logging_enabled) {
        		Log.d(TAG, "Upgrading database from version " + oldVersion + " to " + newVersion + ", which will destroy all old data");
        	}

            db.execSQL("DROP TABLE IF EXISTS players;");
            db.execSQL("DROP TABLE IF EXISTS games");
            db.execSQL("DROP TABLE IF EXISTS game_players");
            db.execSQL("DROP TABLE IF EXISTS turns");
            onCreate(db);
        }
    }

    /**
     * Open a connection to the SQLite database.
     */
    public DB open() throws SQLException 
    {
    	if (logging_enabled) { Log.w(TAG, "Attempting to open db..."); }
        db = DBHelper.getWritableDatabase();
        if (logging_enabled) { Log.w(TAG, "db Should be open now..."); }
        
        return this;
    }

    /**
     * Close the SQLite database connection.
     */
    public void close() 
    {
        DBHelper.close();
    }

    /**
     * Methods to manage SQLite records containing chalkup players.
     * 
     * insertPlayer(String name, String description)
     * deletePlayer(long rowId)
     * getAllPlayers()
     * getPlayer(long rowId)
     * updatePlayer(long rowId, String name, String description)
     */

    /**
     * Insert a player and return the new players ._id
     */
    public long insertPlayer(String name, String description) {
    	ContentValues initialValues = new ContentValues();
    	initialValues.put("name", name);
    	initialValues.put("description", description);
		return db.insert("players", null, initialValues);
    }

    /**
     * Delete player by players._id
     */
    public boolean deletePlayer(long rowId) {
    	return db.delete("players", "_id = " + rowId, null) > 0;
    }

    /**
     * Returns a cursor via rawQuery containing all players records.
     */
    public Cursor getAllPlayers() {
    	return db.rawQuery("select _id, name, description from players order by name", null);
    }

    /**
     * Returns a players data as a cursor by players._id
     */
    public Cursor getPlayer(long rowId) throws SQLException {
    	Cursor mCursor = db.rawQuery("select _id, name, description from players where _id = " + Long.toString(rowId), null);
        if (mCursor != null) {
            mCursor.moveToFirst();
        }
        return mCursor;
    }

    /**
     * Update a players data by players._id
     */
    public boolean updatePlayer(long rowId, String name, String description) {
        ContentValues args = new ContentValues();
        args.put("name", name);
        args.put("description", description);
        return db.update("players", args, "_id = " + rowId, null) > 0;
    }

    /**
     * Methods to manage SQLite records containing chalkup games.
     * 
     * insertGame(String name, String description)
     * deleteGame(long rowId)
     * getAllGames()
     * getGame(long rowId)
     * updateGame(long rowId, String name, String description)
     *    
     */

    public long insertGame( String listType, String name, String description, ArrayList<Long> selected_ids ) {

    	long game_id = 0;

    	// Add the game and get its ID
    	ContentValues initialValues = new ContentValues();
    	initialValues.put("name", name);
    	initialValues.put("description", description);
		game_id = db.insert("games", null, initialValues);

		// Add players

    	if (listType.equals("players")) {
    		// immediately add all the listed players without an associated team.
        	if ( !selected_ids.isEmpty() ) {
        		Log.w(TAG, "There were player IDs specified loop over them.");

        		for(int i = 0 ; i < selected_ids.size() ; i++)
        		{
        			Log.d(TAG, selected_ids.get(i).toString());

        			ContentValues pl = new ContentValues();
        			pl.put("game_id", game_id);
        			pl.put("player_id", selected_ids.get(i));
        			pl.put("player_name", this.getPlayer(selected_ids.get(i)).getPosition()); /***** NEED THIS!!!!! ******/
        			db.insert("game_players", null, pl);

        		}

        	} else {
        		Log.w(TAG, "NO PLAYER IDs specified, just create the game and return its _id");
        	}
    	}

    	return game_id;
    }

    /**
     * Delete a game by games._id
     * Note: This table has a relationship, so we need to delete
     * not only the record in games, but also any record in game_players
     * that are associated with it.
     */
    public boolean deleteGame(long rowId) { 
    	boolean return_value = true;
    	return_value = db.delete("turns", "game_players_id in (select _id from game_players where game_id = " + rowId + ")", null) > 0;
		return_value = db.delete("game_players", "game_id = " + rowId, null) > 0;
		return_value = db.delete("games", "_id = " + rowId, null) > 0;
		return return_value;
    }

    /**
     * Returns a cursor via rawQuery containing all games records.
     */
    public Cursor getAllGames() { 
    	Log.d(TAG, "getAllGames()");
    	return db.rawQuery("select _id, name, dt from games order by _id desc", null);
    }

    /**
     * Returns a game data as a cursor by teams._id
     */
    public Cursor getGame(long rowId) throws SQLException {
        Cursor mCursor = db.query(true, "games", new String[] { "_id", "name", "description" }, "_id = " + rowId, null, null, null, null, null);
        if (mCursor != null) {
            mCursor.moveToFirst();
        }
        return mCursor;
    }

    /************************
     * GAME PLAYERS
     ************************/
    /**
     * Returns a cursor via rawQuery containing all games records.
     */
    public Cursor getAllGamePlayers(long rowId) throws SQLException { 
    	Log.d(TAG, "getAllGamePlayers()");

    	Cursor mCursor = db.rawQuery("select gp._id, p.name, sum(t.change) score from players p left outer join game_players gp on p._id = gp.player_id left outer join games g on g._id = gp.game_id left outer join turns t on t.game_players_id = gp._id where g._id = " + Long.toString(rowId) + " group by p._id", null);
    	if (mCursor != null) { mCursor.moveToFirst(); }
    	return mCursor;

    }

    /* GAME TURNS */

    /* Table: game_turns 
     * _id integer primary key
     * dt TIMESTAMP NOT NULL DEFAULT current_timestamp
     * game_id integer
     * player_name string -- statically store player name so 
     * team_name string
     * score integer   
     */

    // log a turn
    public long turn(long game_players_id, long change) {
    	ContentValues args = new ContentValues();
    	args.put("game_players_id", game_players_id);
    	args.put("change", change);
		return db.insert("turns", null, args);
    }

}
